@@ -1,6 +1,4 @@
 /* -*- Mode: C; tab-width: 8; indent-tabs-mode: t; c-basic-offset: 8 -*-
- *
- *  arch-tag: Implementation of metadata reading using GStreamer
  *
  *  Copyright (C) 2003,2004 Colin Walters <walters@verbum.org>
  *
@@ -53,8 +51,6 @@ static void rb_metadata_finalize (GObject *object);
 
 struct RBMetaDataPrivate
 {
-	char *uri;
-
 	GHashTable *metadata;
 
 	GstElement *pipeline;
@@ -75,6 +71,45 @@ struct RBMetaDataPrivate
 
 #define RB_METADATA_GET_PRIVATE(o) (G_TYPE_INSTANCE_GET_PRIVATE ((o), RB_TYPE_METADATA, RBMetaDataPrivate))
 
+void
+rb_metadata_reset (RBMetaData *md)
+{
+	if (md->priv->metadata)
+		g_hash_table_destroy (md->priv->metadata);
+	md->priv->metadata = g_hash_table_new_full (g_direct_hash, g_direct_equal,
+						    NULL, (GDestroyNotify) rb_value_free);
+
+	if (md->priv->pipeline) {
+		gst_object_unref (md->priv->pipeline);
+		md->priv->pipeline = NULL;
+	}
+	if (md->priv->sink) {
+		md->priv->sink = NULL;
+	}
+	md->priv->typefind_cb_id = 0;
+	if (md->priv->tags) {
+		gst_tag_list_free (md->priv->tags);
+		md->priv->tags = NULL;
+	}
+
+	g_free (md->priv->type);
+	md->priv->type = NULL;
+	md->priv->eos = FALSE;
+	md->priv->has_audio = FALSE;
+	md->priv->has_non_audio = FALSE;
+	md->priv->has_video = FALSE;
+	if (md->priv->missing_plugins != NULL) {
+		GSList *t;
+		for (t = md->priv->missing_plugins; t != NULL; t = t->next) {
+			gst_message_unref (GST_MESSAGE (t->data));
+		}
+		g_slist_free (md->priv->missing_plugins);
+		md->priv->missing_plugins = NULL;
+	}
+
+	g_clear_error (&md->priv->error);
+}
+
 
 static GstElement *
 flac_tagger (GstElement *pipeline, GstElement *link_to, GstTagList *tags)
@@ -87,6 +122,7 @@ flac_tagger (GstElement *pipeline, GstElement *link_to, GstTagList *tags)
 
 	gst_bin_add (GST_BIN (pipeline), tagger);
 	gst_element_link_many (link_to, tagger, NULL);
+	gst_element_set_state (tagger, GST_STATE_PAUSED);
 
 	gst_tag_setter_merge_tags (GST_TAG_SETTER (tagger), tags, GST_TAG_MERGE_REPLACE_ALL);
 	return tagger;
@@ -121,6 +157,8 @@ id3_tagger (GstElement *pipeline, GstElement *link_to, GstTagList *tags)
 		goto error;
 
 	g_signal_connect (demux, "pad-added", (GCallback)id3_pad_added_cb, mux);
+	gst_element_set_state (demux, GST_STATE_PAUSED);
+	gst_element_set_state (mux, GST_STATE_PAUSED);
 
 	gst_tag_setter_merge_tags (GST_TAG_SETTER (mux), tags, GST_TAG_MERGE_REPLACE_ALL);
 	return mux;
@@ -206,6 +244,7 @@ vorbis_tagger (GstElement *pipeline, GstElement *link_to, GstTagList *tags)
 	if (!gst_element_link (link_to, demux))
 		goto error;
 
+	gst_element_set_state (demux, GST_STATE_PAUSED);
 	g_object_set_data (G_OBJECT (demux), "mux", mux);
 	g_signal_connect (demux, "pad-added", (GCallback)ogg_pad_added_cb, tags);
 	return mux;
@@ -244,6 +283,8 @@ mp4_tagger (GstElement *pipeline, GstElement *link_to, GstTagList *tags)
 
 	muxpad = gst_element_get_request_pad (mux, "audio_%d");
 	g_signal_connect (demux, "pad-added", G_CALLBACK (mp4_pad_added_cb), muxpad);
+	gst_element_set_state (demux, GST_STATE_PAUSED);
+	gst_element_set_state (mux, GST_STATE_PAUSED);
 
 	gst_tag_setter_merge_tags (GST_TAG_SETTER (mux), tags, GST_TAG_MERGE_REPLACE_ALL);
 
@@ -316,18 +357,10 @@ rb_metadata_finalize (GObject *object)
 
 	md = RB_METADATA (object);
 
-	if (md->priv->metadata)
-		g_hash_table_destroy (md->priv->metadata);
-
-	if (md->priv->pipeline)
-		gst_object_unref (GST_OBJECT (md->priv->pipeline));
-
+	rb_metadata_reset (md);
 	if (md->priv->taggers)
 		g_hash_table_destroy (md->priv->taggers);
 
-	g_free (md->priv->type);
-	g_free (md->priv->uri);
-	g_clear_error (&md->priv->error);
 
 	G_OBJECT_CLASS (rb_metadata_parent_class)->finalize (object);
 }
@@ -457,6 +490,7 @@ rb_metadata_gst_typefind_cb (GstElement *typefind, guint probability, GstCaps *c
 	}
 
 	g_signal_handler_disconnect (typefind, md->priv->typefind_cb_id);
+	md->priv->typefind_cb_id = 0;
 }
 
 static void
@@ -691,32 +725,11 @@ rb_metadata_load (RBMetaData *md,
 	int change_timeout;
 	GstBus *bus;
 
-	g_free (md->priv->uri);
-	md->priv->uri = NULL;
-	g_free (md->priv->type);
-	md->priv->type = NULL;
-	md->priv->error = NULL;
-	md->priv->eos = FALSE;
-	md->priv->has_audio = FALSE;
-	md->priv->has_non_audio = FALSE;
-	md->priv->has_video = FALSE;
-	md->priv->missing_plugins = NULL;
-
-	if (md->priv->pipeline) {
-		gst_object_unref (GST_OBJECT (md->priv->pipeline));
-		md->priv->pipeline = NULL;
-	}
-
+	rb_metadata_reset (md);
 	if (uri == NULL)
 		return;
 
 	rb_debug ("loading metadata for uri: %s", uri);
-	md->priv->uri = g_strdup (uri);
-
-	if (md->priv->metadata)
-		g_hash_table_destroy (md->priv->metadata);
-	md->priv->metadata = g_hash_table_new_full (g_direct_hash, g_direct_equal,
-						    NULL, (GDestroyNotify) rb_value_free);
 
 	/* The main tagfinding pipeline looks like this:
  	 * <src> ! decodebin ! fakesink
@@ -915,7 +928,7 @@ rb_metadata_gst_add_tag_data (gpointer key, const GValue *val, RBMetaData *md)
 }
 
 static gboolean
-rb_metadata_file_valid (char *original, char *newfile)
+rb_metadata_file_valid (const char *original, const char *newfile)
 {
 	RBMetaData *md = rb_metadata_new ();
 	GError *error = NULL;
@@ -932,25 +945,75 @@ rb_metadata_file_valid (char *original, char *newfile)
 	return ret;
 }
 
+static void
+metadata_save_type_found (GstElement *typefind, guint probability, GstCaps *caps, RBMetaData *md)
+{
+	RBAddTaggerElem add_tagger_func;
+        GstElement *retag_end;
+	GstMessage *message;
+	const char *type;
+	GError *error = NULL;
+
+	g_signal_handler_disconnect (typefind, md->priv->typefind_cb_id);
+	md->priv->typefind_cb_id = 0;
+
+	if (gst_caps_is_empty (caps) || gst_caps_is_any (caps)) {
+		rb_debug ("got empty/no caps from typefind");
+		g_set_error (&error,
+			     RB_METADATA_ERROR,
+			     RB_METADATA_ERROR_UNSUPPORTED,
+			     _("Unable to identify file type"));
+		goto out_error;
+	}
+
+	type = gst_structure_get_name (gst_caps_get_structure (caps, 0));
+
+	/* Tagger element(s) */
+	add_tagger_func = g_hash_table_lookup (md->priv->taggers, type);
+	if (!add_tagger_func) {
+		rb_debug ("found unsupported type %s", type);
+		g_set_error (&error,
+			     RB_METADATA_ERROR,
+			     RB_METADATA_ERROR_UNSUPPORTED,
+			     _("Unsupported file type: %s"), type);
+		goto out_error;
+	}
+
+	rb_debug ("adding tagger for type %s", type);
+	retag_end = add_tagger_func (md->priv->pipeline, typefind, md->priv->tags);
+	if (!retag_end) {
+		g_set_error (&error,
+			     RB_METADATA_ERROR,
+			     RB_METADATA_ERROR_UNSUPPORTED,
+			     _("Unable to create tag-writing elements"));
+		goto out_error;
+	}
+
+	gst_element_link_many (retag_end, md->priv->sink, NULL);
+	return;
+
+out_error:
+	message = gst_message_new_error (GST_OBJECT (typefind), error, NULL);
+	gst_element_post_message (typefind, message);
+	g_clear_error (&error);
+}
+
 void
-rb_metadata_save (RBMetaData *md, GError **error)
+rb_metadata_save (RBMetaData *md, const char *uri, GError **error)
 {
 	GstElement *pipeline = NULL;
 	GstElement *source = NULL;
-        GstElement *retag_end = NULL; /* the last element after retagging subpipeline */
+	GstElement *typefind = NULL;
 	const char *plugin_name = NULL;
 	char *tmpname_prefix = NULL;
 	char *tmpname = NULL;
 	GOutputStream *stream = NULL;
 	GError *io_error = NULL;
-	RBAddTaggerElem add_tagger_func;
 
-	g_return_if_fail (md->priv->uri != NULL);
-	g_return_if_fail (md->priv->type != NULL);
+	g_return_if_fail (uri != NULL);
+	rb_debug ("saving metadata for uri: %s", uri);
 
-	rb_debug ("saving metadata for uri: %s", md->priv->uri);
-
-	tmpname_prefix = rb_uri_make_hidden (md->priv->uri);
+	tmpname_prefix = rb_uri_make_hidden (uri);
 	rb_debug ("temporary file name prefix: %s", tmpname_prefix);
 
 	rb_uri_mkstemp (tmpname_prefix, &tmpname, &stream, &io_error);
@@ -963,46 +1026,39 @@ rb_metadata_save (RBMetaData *md, GError **error)
 	md->priv->pipeline = pipeline;
 
 	/* Source */
-	source = gst_element_make_from_uri (GST_URI_SRC, md->priv->uri, "urisrc");
+	source = gst_element_make_from_uri (GST_URI_SRC, uri, "urisrc");
 	if (source == NULL) {
 		plugin_name = "urisrc";
 		goto missing_plugin;	
 	}
 	gst_bin_add (GST_BIN (pipeline), source);
 
+	/* typefind */
+	plugin_name = "typefind";
+	typefind = gst_element_factory_make (plugin_name, NULL);
+	if (typefind == NULL)
+		goto missing_plugin;
+	gst_bin_add (GST_BIN (pipeline), typefind);
+	md->priv->typefind_cb_id = g_signal_connect_object (typefind,
+							    "have_type",
+							    G_CALLBACK (metadata_save_type_found),
+							    md,
+							    0);
+
 	/* Sink */
 	plugin_name = "giostreamsink";
-	if (!(md->priv->sink = gst_element_factory_make (plugin_name, plugin_name)))
+	if (!(md->priv->sink = gst_element_factory_make (plugin_name, NULL)))
 		goto missing_plugin;
 
-	g_object_set (G_OBJECT (md->priv->sink), "stream", stream, NULL);
+	g_object_set (md->priv->sink, "stream", stream, NULL);
 
 	md->priv->tags = gst_tag_list_new ();
 	g_hash_table_foreach (md->priv->metadata,
 			      (GHFunc) rb_metadata_gst_add_tag_data,
 			      md);
 
-	/* Tagger element(s) */
-	add_tagger_func = g_hash_table_lookup (md->priv->taggers, md->priv->type);
-	if (!add_tagger_func) {
-		g_set_error (error,
-			     RB_METADATA_ERROR,
-			     RB_METADATA_ERROR_UNSUPPORTED,
-			     _("Unsupported file type: %s"), md->priv->type);
-		goto out_error;
-	}
-
-	retag_end = add_tagger_func (md->priv->pipeline, source, md->priv->tags);
-	if (!retag_end) {
-		g_set_error (error,
-			     RB_METADATA_ERROR,
-			     RB_METADATA_ERROR_UNSUPPORTED,
-			     _("Unable to create tag-writing elements"));
-		goto out_error;
-	}
-
 	gst_bin_add (GST_BIN (pipeline), md->priv->sink);
-	gst_element_link_many (retag_end, md->priv->sink, NULL);
+	gst_element_link (source, typefind);
 
 	gst_element_set_state (pipeline, GST_STATE_PLAYING);
 
@@ -1032,7 +1088,7 @@ rb_metadata_save (RBMetaData *md, GError **error)
 		stream = NULL;
 
 		/* check to ensure the file isn't corrupt */
-		if (!rb_metadata_file_valid (md->priv->uri, tmpname)) {
+		if (!rb_metadata_file_valid (uri, tmpname)) {
 			g_set_error (error,
 				     RB_METADATA_ERROR,
 				     RB_METADATA_ERROR_INTERNAL,
@@ -1041,7 +1097,7 @@ rb_metadata_save (RBMetaData *md, GError **error)
 		}
 
 		src = g_file_new_for_uri (tmpname);
-		dest = g_file_new_for_uri (md->priv->uri);
+		dest = g_file_new_for_uri (uri);
 		g_file_move (src, dest, G_FILE_COPY_OVERWRITE, NULL, NULL, NULL, &io_error);
 		if (io_error != NULL) {
 			goto gio_error;
