@@ -31,6 +31,8 @@
 #include <glib.h>
 #include <rb-source.h>
 
+#include "mediaplayerid.h"
+
 G_BEGIN_DECLS
 gboolean rb_ipod_helpers_show_first_time_dialog (GMount *mount,
 						 const char *builder_file);
@@ -38,7 +40,7 @@ gboolean rb_ipod_helpers_show_first_time_dialog (GMount *mount,
 guint64 rb_ipod_helpers_get_capacity (const char *mountpoint);
 guint64 rb_ipod_helpers_get_free_space (const char *mountpoint);
 char *rb_ipod_helpers_get_device (RBSource *source);
-gboolean rb_ipod_helpers_is_ipod (GMount *mount);
+gboolean rb_ipod_helpers_is_ipod (GMount *mount, MPIDDevice *device_info);
 gboolean rb_ipod_helpers_needs_init (GMount *mount);
 G_END_DECLS
 
