@@ -1,6 +1,4 @@
 /*
- *  arch-tag: Implementation of ipod source object
- *
  *  Copyright (C) 2004, 2007 Christophe Fergeau  <teuf@gnome.org>
  *
  *  This program is free software; you can redistribute it and/or modify
@@ -50,18 +48,23 @@
 #include "rb-util.h"
 #include "rhythmdb.h"
 #include "rb-cut-and-paste-code.h"
+#include "rb-media-player-source.h"
+#include "rb-playlist-source.h"
+#include "rb-playlist-manager.h"
+#include "rb-podcast-manager.h"
 
 #define CONF_STATE_PANED_POSITION CONF_PREFIX "/state/ipod/paned_position"
 #define CONF_STATE_SHOW_BROWSER   CONF_PREFIX "/state/ipod/show_browser"
 
 static void rb_ipod_source_constructed (GObject *object);
+static void rb_ipod_source_class_init (RBiPodSourceClass *klass);
 static void rb_ipod_source_dispose (GObject *object);
 
 static char *impl_get_browser_key (RBSource *source);
 static char *impl_get_paned_key (RBBrowserSource *source);
 
 static gboolean impl_show_popup (RBSource *source);
-static void impl_move_to_trash (RBSource *asource);
+static void impl_delete (RBSource *asource);
 static void rb_ipod_load_songs (RBiPodSource *source);
 static void impl_delete_thyself (RBSource *source);
 static GList* impl_get_ui_actions (RBSource *source);
@@ -70,6 +73,7 @@ static GList * impl_get_mime_types (RBRemovableMediaSource *source);
 static gboolean impl_track_added (RBRemovableMediaSource *source,
 				  RhythmDBEntry *entry,
 				  const char *dest,
+				  guint64 filesize,
 				  const char *mimetype);
 static char* impl_build_dest_uri (RBRemovableMediaSource *source,
 				  RhythmDBEntry *entry,
@@ -87,6 +91,19 @@ static gboolean rb_ipod_song_artwork_add_cb (RhythmDB *db,
                                              const GValue *metadata,
                                              RBiPodSource *isource);
 
+static guint64 impl_get_capacity (RBMediaPlayerSource *source);
+static guint64 impl_get_free_space (RBMediaPlayerSource *source);
+static void impl_show_properties (RBMediaPlayerSource *source, GtkWidget *info_box, GtkWidget *notebook);
+
+static void rb_ipod_source_set_property (GObject *object,
+					 guint prop_id,
+					 const GValue *value,
+					 GParamSpec *pspec);
+static void rb_ipod_source_get_property (GObject *object,
+					 guint prop_id,
+					 GValue *value,
+					 GParamSpec *pspec);
+
 static RhythmDB *get_db_for_source (RBiPodSource *source);
 
 struct _PlayedEntry {
@@ -101,6 +118,8 @@ typedef struct
 	RbIpodDb *ipod_db;
 	GHashTable *entry_map;
 
+	MPIDDevice *device_info;
+
 	gboolean needs_shuffle_db;
 	RBIpodStaticPlaylistSource *podcast_pl;
 
@@ -117,9 +136,16 @@ typedef struct {
 	GdkPixbuf *pixbuf;
 } RBiPodSongArtworkAddData;
 
+enum
+{
+	PROP_0,
+	PROP_DEVICE_INFO,
+	PROP_DEVICE_SERIAL
+};
+
 RB_PLUGIN_DEFINE_TYPE(RBiPodSource,
 		      rb_ipod_source,
-		      RB_TYPE_REMOVABLE_MEDIA_SOURCE)
+		      RB_TYPE_MEDIA_PLAYER_SOURCE)
 
 #define IPOD_SOURCE_GET_PRIVATE(o)   (G_TYPE_INSTANCE_GET_PRIVATE ((o), RB_TYPE_IPOD_SOURCE, RBiPodSourcePrivate))
 
@@ -128,22 +154,32 @@ rb_ipod_source_class_init (RBiPodSourceClass *klass)
 {
 	GObjectClass *object_class = G_OBJECT_CLASS (klass);
 	RBSourceClass *source_class = RB_SOURCE_CLASS (klass);
+	RBMediaPlayerSourceClass *mps_class = RB_MEDIA_PLAYER_SOURCE_CLASS (klass);
 	RBRemovableMediaSourceClass *rms_class = RB_REMOVABLE_MEDIA_SOURCE_CLASS (klass);
 	RBBrowserSourceClass *browser_source_class = RB_BROWSER_SOURCE_CLASS (klass);
 
 	object_class->constructed = rb_ipod_source_constructed;
 	object_class->dispose = rb_ipod_source_dispose;
 
+	object_class->set_property = rb_ipod_source_set_property;
+	object_class->get_property = rb_ipod_source_get_property;
+
 	source_class->impl_can_browse = (RBSourceFeatureFunc) rb_true_function;
 	source_class->impl_get_browser_key  = impl_get_browser_key;
 	source_class->impl_show_popup = impl_show_popup;
 	source_class->impl_delete_thyself = impl_delete_thyself;
-	source_class->impl_can_move_to_trash = (RBSourceFeatureFunc) rb_true_function;
-	source_class->impl_move_to_trash = impl_move_to_trash;
+	source_class->impl_can_move_to_trash = (RBSourceFeatureFunc) rb_false_function;
 	source_class->impl_can_rename = (RBSourceFeatureFunc) rb_true_function;
 	source_class->impl_get_ui_actions = impl_get_ui_actions;
+	source_class->impl_can_delete = (RBSourceFeatureFunc) rb_true_function;
+	source_class->impl_delete = impl_delete;
 
 	source_class->impl_can_paste = (RBSourceFeatureFunc) rb_true_function;
+
+	mps_class->impl_get_capacity = impl_get_capacity;
+	mps_class->impl_get_free_space = impl_get_free_space;
+	mps_class->impl_show_properties = impl_show_properties;
+
 	rms_class->impl_should_paste = rb_removable_media_source_should_paste_no_duplicate;
 	rms_class->impl_track_added = impl_track_added;
 	rms_class->impl_build_dest_uri = impl_build_dest_uri;
@@ -151,9 +187,60 @@ rb_ipod_source_class_init (RBiPodSourceClass *klass)
 
 	browser_source_class->impl_get_paned_key = impl_get_paned_key;
 
+	g_object_class_install_property (object_class,
+					 PROP_DEVICE_INFO,
+					 g_param_spec_object ("device-info",
+							      "device info",
+							      "device information object",
+							      MPID_TYPE_DEVICE,
+							      G_PARAM_READWRITE | G_PARAM_CONSTRUCT_ONLY));
+	g_object_class_override_property (object_class, PROP_DEVICE_SERIAL, "serial");
+
 	g_type_class_add_private (klass, sizeof (RBiPodSourcePrivate));
 }
 
+static void
+rb_ipod_source_set_property (GObject *object,
+			     guint prop_id,
+			     const GValue *value,
+			     GParamSpec *pspec)
+{
+	RBiPodSourcePrivate *priv = IPOD_SOURCE_GET_PRIVATE (object);
+
+	switch (prop_id) {
+	case PROP_DEVICE_INFO:
+		priv->device_info = g_value_dup_object (value);
+		break;
+	default:
+		G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
+		break;
+	}
+}
+
+static void
+rb_ipod_source_get_property (GObject *object,
+			     guint prop_id,
+			     GValue *value,
+			     GParamSpec *pspec)
+{
+	RBiPodSourcePrivate *priv = IPOD_SOURCE_GET_PRIVATE (object);
+
+	switch (prop_id) {
+	case PROP_DEVICE_INFO:
+		g_value_set_object (value, priv->device_info);
+		break;
+	case PROP_DEVICE_SERIAL:
+		{
+			char *serial;
+			g_object_get (priv->device_info, "serial", &serial, NULL);
+			g_value_take_string (value, serial);
+		}
+		break;
+	default:
+		G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
+		break;
+	}
+}
 
 static void
 rb_ipod_source_set_ipod_name (RBiPodSource *source, const char *name)
@@ -251,10 +338,11 @@ rb_ipod_source_dispose (GObject *object)
 	G_OBJECT_CLASS (rb_ipod_source_parent_class)->dispose (object);
 }
 
-RBRemovableMediaSource *
+RBMediaPlayerSource *
 rb_ipod_source_new (RBPlugin *plugin,
 		    RBShell *shell,
-		    GMount *mount)
+		    GMount *mount,
+		    MPIDDevice *device_info)
 {
 	RBiPodSource *source;
 	RhythmDBEntryType entry_type;
@@ -284,12 +372,13 @@ rb_ipod_source_new (RBPlugin *plugin,
 					       "mount", mount,
 					       "shell", shell,
 					       "source-group", RB_SOURCE_GROUP_DEVICES,
+					       "device-info", device_info,
 					       NULL));
 
 	rb_shell_register_entry_type_for_source (shell, RB_SOURCE (source), entry_type);
         g_boxed_free (RHYTHMDB_TYPE_ENTRY_TYPE, entry_type);
 
-	return RB_REMOVABLE_MEDIA_SOURCE (source);
+	return RB_MEDIA_PLAYER_SOURCE (source);
 }
 
 static void
@@ -449,7 +538,7 @@ load_ipod_playlists (RBiPodSource *source)
 #define MEDIATYPE_PODCAST       0x0004
 
 static Itdb_Track *
-create_ipod_song_from_entry (RhythmDBEntry *entry, const char *mimetype)
+create_ipod_song_from_entry (RhythmDBEntry *entry, guint64 filesize, const char *mimetype)
 {
 	Itdb_Track *track;
 
@@ -464,7 +553,7 @@ create_ipod_song_from_entry (RhythmDBEntry *entry, const char *mimetype)
 	                                                RHYTHMDB_PROP_ALBUM_SORTNAME);
 	track->genre = rhythmdb_entry_dup_string (entry, RHYTHMDB_PROP_GENRE);
 	track->filetype = g_strdup (mimetype);
-	track->size = rhythmdb_entry_get_uint64 (entry, RHYTHMDB_PROP_FILE_SIZE);
+	track->size = filesize;
 	track->tracklen = rhythmdb_entry_get_ulong (entry, RHYTHMDB_PROP_DURATION);
 	track->tracklen *= 1000;
 	track->cd_nr = rhythmdb_entry_get_ulong (entry, RHYTHMDB_PROP_DISC_NUMBER);
@@ -976,7 +1065,7 @@ impl_show_popup (RBSource *source)
 }
 
 void
-rb_ipod_source_trash_entries (RBiPodSource *source, GList *entries)
+rb_ipod_source_delete_entries (RBiPodSource *source, GList *entries)
 {
 	RBiPodSourcePrivate *priv = IPOD_SOURCE_GET_PRIVATE (source);
 	RhythmDB *db;
@@ -1011,14 +1100,14 @@ rb_ipod_source_trash_entries (RBiPodSource *source, GList *entries)
 }
 
 static void
-impl_move_to_trash (RBSource *source)
+impl_delete (RBSource *source)
 {
 	GList *sel;
 	RBEntryView *songs;
 
 	songs = rb_source_get_entry_view (source);
 	sel = rb_entry_view_get_selected_entries (songs);
-	rb_ipod_source_trash_entries (RB_IPOD_SOURCE (source), sel);
+	rb_ipod_source_delete_entries (RB_IPOD_SOURCE (source), sel);
 
 	g_list_foreach (sel, (GFunc) rhythmdb_entry_unref, NULL);
 	g_list_free (sel);
@@ -1088,6 +1177,7 @@ rb_add_artwork_whole_album_cb (GtkTreeModel *query_model,
 	entry = rhythmdb_query_model_iter_to_entry (RHYTHMDB_QUERY_MODEL (query_model), iter);
 
 	song = g_hash_table_lookup (artwork_data->priv->entry_map, entry);
+	rhythmdb_entry_unref (entry);
 	g_return_val_if_fail (song != NULL, FALSE);
 
 	if (song->has_artwork == 0x01) {		   
@@ -1220,6 +1310,7 @@ static gboolean
 impl_track_added (RBRemovableMediaSource *source,
 		  RhythmDBEntry *entry,
 		  const char *dest,
+		  guint64 filesize,
 		  const char *mimetype)
 {
 	RBiPodSource *isource = RB_IPOD_SOURCE (source);
@@ -1228,7 +1319,7 @@ impl_track_added (RBRemovableMediaSource *source,
 
 	db = get_db_for_source (isource);
 
-	song = create_ipod_song_from_entry (entry, mimetype);
+	song = create_ipod_song_from_entry (entry, filesize, mimetype);
 	if (song != NULL) {
 		RBiPodSourcePrivate *priv = IPOD_SOURCE_GET_PRIVATE (source);
 		char *filename;
@@ -1590,17 +1681,6 @@ rb_ipod_source_remove_playlist (RBiPodSource *ipod_source,
 	rb_ipod_db_remove_playlist (priv->ipod_db, rb_ipod_static_playlist_source_get_itdb_playlist (playlist_source));
 }
 
-static void
-rb_ipod_info_response_cb (GtkDialog *dialog,
- 			  int response_id,
- 			  RBiPodSource *source)
-{
-	if (response_id == GTK_RESPONSE_CLOSE) {
- 		
-		gtk_widget_destroy (GTK_WIDGET (dialog));
-	}
-}
-
 static gboolean
 ipod_name_changed_cb (GtkWidget     *widget,
  		      GdkEventFocus *event,
@@ -1612,28 +1692,32 @@ ipod_name_changed_cb (GtkWidget     *widget,
 	return FALSE;
 }
 
-void
-rb_ipod_source_show_properties (RBiPodSource *source)
+
+static void
+impl_show_properties (RBMediaPlayerSource *source, GtkWidget *info_box, GtkWidget *notebook)
 {
+	RBiPodSourcePrivate *priv = IPOD_SOURCE_GET_PRIVATE (source);
+	GHashTableIter iter;
+	int num_podcasts;
+	gpointer key, value;
 	GtkBuilder *builder;
-	GObject *dialog;
-	GObject *label;
+	GtkWidget *widget;
 	char *text;
 	const gchar *mp;
-	char *used;
-	char *capacity;
 	char *builder_file;
- 	RBiPodSourcePrivate *priv = IPOD_SOURCE_GET_PRIVATE (source);
 	Itdb_Device *ipod_dev;
 	RBPlugin *plugin;
+	GList *output_formats;
+	GList *t;
+	GString *str;
+
+	/* probably should display an error on the basic page in most of these error cases.. */
 
 	if (priv->ipod_db == NULL) {
 		rb_debug ("can't show ipod properties with no ipod db");
 		return;
 	}
 
-	ipod_dev = rb_ipod_db_get_device (priv->ipod_db);
-
 	g_object_get (source, "plugin", &plugin, NULL);
 	builder_file = rb_plugin_find_file (plugin, "ipod-info.ui");
 	g_object_unref (plugin);
@@ -1651,66 +1735,105 @@ rb_ipod_source_show_properties (RBiPodSource *source)
  		return;
  	}
 	
- 	dialog = gtk_builder_get_object (builder, "ipod-information");
- 	g_signal_connect_object (dialog,
- 				 "response",
- 				 G_CALLBACK (rb_ipod_info_response_cb),
- 				 source, 0);
- 
- 	label = gtk_builder_get_object (builder, "label-number-track-number");
- 	text = g_strdup_printf ("%u", g_list_length (rb_ipod_db_get_tracks(priv->ipod_db) ));
- 	gtk_label_set_text (GTK_LABEL (label), text);
- 	g_free (text);
- 
- 	label = gtk_builder_get_object (builder, "entry-ipod-name");
- 	gtk_entry_set_text (GTK_ENTRY (label), rb_ipod_db_get_ipod_name (priv->ipod_db));
- 	g_signal_connect (label, "focus-out-event",
+	ipod_dev = rb_ipod_db_get_device (priv->ipod_db);
+
+	/* basic tab stuff */
+	widget = GTK_WIDGET (gtk_builder_get_object (builder, "ipod-basic-info"));
+	gtk_box_pack_start (GTK_BOX (info_box), widget, TRUE, TRUE, 0);
+
+	widget = GTK_WIDGET (gtk_builder_get_object (builder, "ipod-name-entry"));
+	gtk_entry_set_text (GTK_ENTRY (widget), rb_ipod_db_get_ipod_name (priv->ipod_db));
+	g_signal_connect (widget, "focus-out-event",
  			  (GCallback)ipod_name_changed_cb, source);
- 
- 	label = gtk_builder_get_object (builder, "label-number-playlist-number");
- 	text = g_strdup_printf ("%u", g_list_length (rb_ipod_db_get_playlists (priv->ipod_db)));
- 	gtk_label_set_text (GTK_LABEL (label), text);
- 	g_free (text);
- 
- 	label = gtk_builder_get_object (builder, "label-mount-point-value");
+
+	num_podcasts = 0;
+	g_hash_table_iter_init (&iter, priv->entry_map);
+	while (g_hash_table_iter_next (&iter, &key, &value)) {
+		Itdb_Track *track = value;
+		if (track->mediatype == MEDIATYPE_PODCAST) {
+			num_podcasts++;
+		}
+	}
+
+	/* TODO these need to be updated as entries are added and removed. */
+	widget = GTK_WIDGET (gtk_builder_get_object (builder, "ipod-num-tracks"));
+	text = g_strdup_printf ("%d", g_hash_table_size (priv->entry_map) - num_podcasts);
+	gtk_label_set_text (GTK_LABEL (widget), text);
+	g_free (text);
+
+	widget = GTK_WIDGET (gtk_builder_get_object (builder, "ipod-num-podcasts"));
+	text = g_strdup_printf ("%d", num_podcasts);
+	gtk_label_set_text (GTK_LABEL (widget), text);
+	g_free (text);
+
+	/* TODO probably needs to ignore the master playlist? */
+	widget = GTK_WIDGET (gtk_builder_get_object (builder, "ipod-num-playlists"));
+	text = g_strdup_printf ("%d", g_list_length (rb_ipod_db_get_playlists (priv->ipod_db)));
+	gtk_label_set_text (GTK_LABEL (widget), text);
+	g_free (text);
+
+	/* 'advanced' tab stuff */
+	widget = GTK_WIDGET (gtk_builder_get_object (builder, "ipod-advanced-tab"));
+	gtk_notebook_append_page (GTK_NOTEBOOK (notebook), widget, gtk_label_new (_("Advanced")));
+
+	widget = GTK_WIDGET (gtk_builder_get_object (builder, "label-mount-point-value"));
 	mp = rb_ipod_db_get_mount_path (priv->ipod_db);
- 	gtk_label_set_text (GTK_LABEL (label), mp);
+	gtk_label_set_text (GTK_LABEL (widget), mp);
 
-	label = gtk_builder_get_object (builder, "progressbar-ipod-usage");
-	used = g_format_size_for_display (rb_ipod_helpers_get_capacity (mp) - rb_ipod_helpers_get_free_space (mp));
-	capacity = g_format_size_for_display (rb_ipod_helpers_get_capacity(mp));
-	gtk_progress_bar_set_fraction (GTK_PROGRESS_BAR (label), 
-				       (double)(rb_ipod_helpers_get_capacity (mp) - rb_ipod_helpers_get_free_space (mp))/(double)rb_ipod_helpers_get_capacity (mp));
-	/* Translators: this is used to display the amount of storage space
-	 * used and the total storage space on an iPod.
-	 */
-	text = g_strdup_printf (_("%s of %s"), used, capacity);
-	gtk_progress_bar_set_text (GTK_PROGRESS_BAR (label), text);
-	g_free (text);
-	g_free (capacity);
-	g_free (used);
-
-	label = gtk_builder_get_object (builder, "label-device-node-value");
+	widget = GTK_WIDGET (gtk_builder_get_object (builder, "label-device-node-value"));
 	text = rb_ipod_helpers_get_device (RB_SOURCE(source));
-	gtk_label_set_text (GTK_LABEL (label), text);
+	gtk_label_set_text (GTK_LABEL (widget), text);
 	g_free (text);
 
- 	label = gtk_builder_get_object (builder, "label-ipod-model-value");
- 	gtk_label_set_text (GTK_LABEL (label), itdb_device_get_sysinfo (ipod_dev, "ModelNumStr"));
+	widget = GTK_WIDGET (gtk_builder_get_object (builder, "label-ipod-model-value"));
+	gtk_label_set_text (GTK_LABEL (widget), itdb_device_get_sysinfo (ipod_dev, "ModelNumStr"));
 
- 	label = gtk_builder_get_object (builder, "label-database-version-value");
+	widget = GTK_WIDGET (gtk_builder_get_object (builder, "label-database-version-value"));
 	text = g_strdup_printf ("%u", rb_ipod_db_get_database_version (priv->ipod_db));
- 	gtk_label_set_text (GTK_LABEL (label), text);
+	gtk_label_set_text (GTK_LABEL (widget), text);
 	g_free (text);
 
- 	label = gtk_builder_get_object (builder, "label-serial-number-value");
-	gtk_label_set_text (GTK_LABEL (label), itdb_device_get_sysinfo (ipod_dev, "pszSerialNumber"));
+	g_object_get (priv->device_info, "serial", &text, NULL);
+	widget = GTK_WIDGET (gtk_builder_get_object (builder, "label-serial-number-value"));
+	gtk_label_set_text (GTK_LABEL (widget), text);
+	g_free (text);
 
- 	label = gtk_builder_get_object (builder, "label-firmware-version-value");
-	gtk_label_set_text (GTK_LABEL (label), itdb_device_get_sysinfo (ipod_dev, "VisibleBuildID"));
+	widget = GTK_WIDGET (gtk_builder_get_object (builder, "label-firmware-version-value"));
+	gtk_label_set_text (GTK_LABEL (widget), itdb_device_get_sysinfo (ipod_dev, "VisibleBuildID"));
 
- 	gtk_widget_show (GTK_WIDGET (dialog));
+	str = g_string_new ("");
+	output_formats = rb_removable_media_source_get_format_descriptions (RB_REMOVABLE_MEDIA_SOURCE (source));
+	for (t = output_formats; t != NULL; t = t->next) {
+		if (t != output_formats) {
+			g_string_append (str, "\n");
+		}
+		g_string_append (str, t->data);
+	}
+	rb_list_deep_free (output_formats);
+
+	widget = GTK_WIDGET (gtk_builder_get_object (builder, "label-audio-formats-value"));
+	gtk_label_set_text (GTK_LABEL (widget), str->str);
+	g_string_free (str, TRUE);
 
 	g_object_unref (builder);
 }
 
+static const gchar *
+get_mount_point	(RBiPodSource *source)
+{
+	RBiPodSourcePrivate *priv = IPOD_SOURCE_GET_PRIVATE (source);
+	return rb_ipod_db_get_mount_path (priv->ipod_db);
+}
+
+static guint64
+impl_get_capacity (RBMediaPlayerSource *source)
+{
+	return rb_ipod_helpers_get_capacity (get_mount_point (RB_IPOD_SOURCE (source)));
+}
+
+static guint64
+impl_get_free_space (RBMediaPlayerSource *source)
+{
+	return rb_ipod_helpers_get_free_space (get_mount_point (RB_IPOD_SOURCE (source)));
+}
+
