@@ -1,6 +1,4 @@
 /*
- *  arch-tag: Implementation of mtp source object
- *
  *  Copyright (C) 2006 Peter Grundström  <pete@openfestis.org>
  *
  *  This program is free software; you can redistribute it and/or modify
@@ -34,11 +32,17 @@
 #include <glib/gi18n.h>
 #include <gst/gst.h>
 
+#if defined(HAVE_GUDEV)
+#define G_UDEV_API_IS_SUBJECT_TO_CHANGE
+#include <gudev/gudev.h>
+#endif
+
 #include "rhythmdb.h"
 #include "eel-gconf-extensions.h"
 #include "rb-debug.h"
 #include "rb-file-helpers.h"
 #include "rb-plugin.h"
+#include "rb-builder-helpers.h"
 #include "rb-removable-media-manager.h"
 #include "rb-static-playlist-source.h"
 #include "rb-util.h"
@@ -52,6 +56,11 @@
 #include "rb-mtp-source.h"
 #include "rb-mtp-thread.h"
 
+#if !GLIB_CHECK_VERSION(2,22,0)
+#define g_mount_unmount_with_operation_finish g_mount_unmount_finish
+#define g_mount_unmount_with_operation(m,f,mo,ca,cb,ud) g_mount_unmount(m,f,ca,cb,ud)
+#endif
+
 #define CONF_STATE_PANED_POSITION CONF_PREFIX "/state/mtp/paned_position"
 #define CONF_STATE_SHOW_BROWSER   CONF_PREFIX "/state/mtp/show_browser"
 
@@ -78,6 +87,7 @@ static GList * impl_get_mime_types (RBRemovableMediaSource *source);
 static gboolean impl_track_added (RBRemovableMediaSource *source,
 				  RhythmDBEntry *entry,
 				  const char *dest,
+				  guint64 filesize,
 				  const char *mimetype);
 static gboolean impl_track_add_error (RBRemovableMediaSource *source,
 				      RhythmDBEntry *entry,
@@ -97,6 +107,10 @@ static void artwork_notify_cb (RhythmDB *db,
 			       const GValue *metadata,
 			       RBMtpSource *source);
 
+static guint64		impl_get_capacity	(RBMediaPlayerSource *source);
+static guint64		impl_get_free_space	(RBMediaPlayerSource *source);
+static void		impl_show_properties	(RBMediaPlayerSource *source, GtkWidget *info_box, GtkWidget *notebook);
+
 static void prepare_player_source_cb (RBPlayer *player,
 				      const char *stream_uri,
 				      GstElement *src,
@@ -109,26 +123,40 @@ static void prepare_encoder_sink_cb (RBEncoderFactory *factory,
 				     const char *stream_uri,
 				     GObject *sink,
 				     RBMtpSource *source);
+#if defined(HAVE_GUDEV)
+static GMount *find_mount_for_device (GUdevDevice *device);
+#endif
 
 typedef struct
 {
 	RBMtpThread *device_thread;
-	LIBMTP_raw_device_t *raw_device;
+	LIBMTP_raw_device_t raw_device;
 	GHashTable *entry_map;
 	GHashTable *artwork_request_map;
 	GHashTable *track_transfer_map;
-#if !defined(HAVE_GUDEV)
+#if defined(HAVE_GUDEV)
+	GUdevDevice *udev_device;
+	GVolume *remount_volume;
+#else
 	char *udi;
 #endif
 	uint16_t supported_types[LIBMTP_FILETYPE_UNKNOWN+1];
 	GList *mediatypes;
 	gboolean album_art_supported;
 
+	/* device information */
+	char *manufacturer;
+	char *serial;
+	char *device_version;
+	char *model_name;
+	guint64 capacity;
+	guint64 free_space;		/* updated by callbacks */
+
 } RBMtpSourcePrivate;
 
 RB_PLUGIN_DEFINE_TYPE(RBMtpSource,
 		       rb_mtp_source,
-		       RB_TYPE_REMOVABLE_MEDIA_SOURCE)
+		       RB_TYPE_MEDIA_PLAYER_SOURCE)
 
 #define MTP_SOURCE_GET_PRIVATE(o) (G_TYPE_INSTANCE_GET_PRIVATE ((o), RB_TYPE_MTP_SOURCE, RBMtpSourcePrivate))
 
@@ -136,7 +164,9 @@ enum
 {
 	PROP_0,
 	PROP_RAW_DEVICE,
+	PROP_UDEV_DEVICE,
 	PROP_UDI,
+	PROP_DEVICE_SERIAL
 };
 
 static void
@@ -146,6 +176,7 @@ rb_mtp_source_class_init (RBMtpSourceClass *klass)
 	RBSourceClass *source_class = RB_SOURCE_CLASS (klass);
 	RBRemovableMediaSourceClass *rms_class = RB_REMOVABLE_MEDIA_SOURCE_CLASS (klass);
 	RBBrowserSourceClass *browser_source_class = RB_BROWSER_SOURCE_CLASS (klass);
+	RBMediaPlayerSourceClass *mps_class = RB_MEDIA_PLAYER_SOURCE_CLASS (klass);
 
 	object_class->constructed = rb_mtp_source_constructed;
 	object_class->dispose = rb_mtp_source_dispose;
@@ -175,13 +206,25 @@ rb_mtp_source_class_init (RBMtpSourceClass *klass)
 	rms_class->impl_get_mime_types = impl_get_mime_types;
 	rms_class->impl_should_paste = rb_removable_media_source_should_paste_no_duplicate;
 
+	mps_class->impl_get_capacity = impl_get_capacity;
+	mps_class->impl_get_free_space = impl_get_free_space;
+	mps_class->impl_show_properties = impl_show_properties;
+
 	g_object_class_install_property (object_class,
 					 PROP_RAW_DEVICE,
 					 g_param_spec_pointer ("raw-device",
 							       "raw-device",
 							       "libmtp raw device",
 							       G_PARAM_READWRITE | G_PARAM_CONSTRUCT_ONLY));
-#if !defined(HAVE_GUDEV)
+#if defined(HAVE_GUDEV)
+	g_object_class_install_property (object_class,
+					 PROP_UDEV_DEVICE,
+					 g_param_spec_object ("udev-device",
+							      "udev-device",
+							      "GUdev device object",
+							      G_UDEV_TYPE_DEVICE,
+							      G_PARAM_READWRITE | G_PARAM_CONSTRUCT_ONLY));
+#else
 	g_object_class_install_property (object_class,
 					 PROP_UDI,
 					 g_param_spec_string ("udi",
@@ -190,6 +233,7 @@ rb_mtp_source_class_init (RBMtpSourceClass *klass)
 							      NULL,
 							      G_PARAM_READWRITE | G_PARAM_CONSTRUCT_ONLY));
 #endif
+	g_object_class_override_property (object_class, PROP_DEVICE_SERIAL, "serial");
 
 	g_type_class_add_private (klass, sizeof (RBMtpSourcePrivate));
 }
@@ -221,6 +265,52 @@ rb_mtp_source_init (RBMtpSource *source)
 	priv->track_transfer_map = g_hash_table_new_full (g_str_hash, g_str_equal, g_free, NULL);
 }
 
+
+static void
+open_device (RBMtpSource *source)
+{
+	RBMtpSourcePrivate *priv = MTP_SOURCE_GET_PRIVATE (source);
+
+	rb_debug ("actually opening device");
+	priv->device_thread = rb_mtp_thread_new ();
+	rb_mtp_thread_open_device (priv->device_thread,
+				   &priv->raw_device,
+				   (RBMtpOpenCallback)mtp_device_open_cb,
+				   g_object_ref (source),
+				   g_object_unref);
+}
+
+#if defined(HAVE_GUDEV)
+static void
+unmount_done_cb (GObject *object, GAsyncResult *result, gpointer psource)
+{
+	GMount *mount;
+	RBMtpSource *source;
+	gboolean ok;
+	GError *error = NULL;
+	RBMtpSourcePrivate *priv;
+
+	mount = G_MOUNT (object);
+	source = RB_MTP_SOURCE (psource);
+	priv = MTP_SOURCE_GET_PRIVATE (source);
+
+	ok = g_mount_unmount_with_operation_finish (mount, result, &error);
+	if (ok) {
+		rb_debug ("successfully unmounted mtp device");
+		priv->remount_volume = g_mount_get_volume (mount);
+
+		open_device (source);
+	} else {
+		g_warning ("Unable to unmount MTP device: %s", error->message);
+		g_error_free (error);
+	}
+
+	g_object_unref (mount);
+	g_object_unref (source);
+}
+
+#endif
+
 static void
 rb_mtp_source_constructed (GObject *object)
 {
@@ -232,15 +322,30 @@ rb_mtp_source_constructed (GObject *object)
 	GObject *player_backend;
 	GtkIconTheme *theme;
 	GdkPixbuf *pixbuf;
+#if defined(HAVE_GUDEV)
+	GMount *mount;
+#endif
 	gint size;
 
 	RB_CHAIN_GOBJECT_METHOD (rb_mtp_source_parent_class, constructed, object);
 	source = RB_MTP_SOURCE (object);
 	priv = MTP_SOURCE_GET_PRIVATE (source);
 
-	/* start the device thread */
-	priv->device_thread = rb_mtp_thread_new ();
-	rb_mtp_thread_open_device (priv->device_thread, priv->raw_device, (RBMtpOpenCallback)mtp_device_open_cb, g_object_ref (source), g_object_unref);
+	/* try to open the device.  if gvfs has mounted it, unmount it first */
+#if defined(HAVE_GUDEV)
+	mount = find_mount_for_device (priv->udev_device);
+	if (mount != NULL) {
+		rb_debug ("device is already mounted, waiting until activated");
+		g_mount_unmount_with_operation (mount,
+						G_MOUNT_UNMOUNT_NONE,
+						NULL,
+						NULL,
+						unmount_done_cb,
+						g_object_ref (source));
+		/* mount gets unreffed in callback */
+	} else
+#endif
+	open_device (source);
 
 	tracks = rb_source_get_entry_view (RB_SOURCE (source));
 	rb_entry_view_append_column (tracks, RB_ENTRY_VIEW_COL_RATING, FALSE);
@@ -293,12 +398,18 @@ rb_mtp_source_set_property (GObject *object,
 			    GParamSpec *pspec)
 {
 	RBMtpSourcePrivate *priv = MTP_SOURCE_GET_PRIVATE (object);
+	LIBMTP_raw_device_t *raw_device;
 
 	switch (prop_id) {
 	case PROP_RAW_DEVICE:
-		priv->raw_device = g_value_get_pointer (value);
+		raw_device = g_value_get_pointer (value);
+		priv->raw_device = *raw_device;
 		break;
-#if !defined(HAVE_GUDEV)
+#if defined(HAVE_GUDEV)
+	case PROP_UDEV_DEVICE:
+		priv->udev_device = g_value_dup_object (value);
+		break;
+#else
 	case PROP_UDI:
 		priv->udi = g_value_dup_string (value);
 		break;
@@ -319,19 +430,44 @@ rb_mtp_source_get_property (GObject *object,
 
 	switch (prop_id) {
 	case PROP_RAW_DEVICE:
-		g_value_set_pointer (value, priv->raw_device);
+		g_value_set_pointer (value, &priv->raw_device);
 		break;
-#if !defined(HAVE_GUDEV)
+#if defined(HAVE_GUDEV)
+	case PROP_UDEV_DEVICE:
+		g_value_set_object (value, priv->udev_device);
+		break;
+#else
 	case PROP_UDI:
 		g_value_set_string (value, priv->udi);
 		break;
 #endif
+	case PROP_DEVICE_SERIAL:
+		g_value_set_string (value, priv->serial);
+		break;
 	default:
 		G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
 		break;
 	}
 }
 
+#if defined(HAVE_GUDEV)
+static void
+remount_done_cb (GObject *object, GAsyncResult *result, gpointer no)
+{
+	gboolean ok;
+	GError *error = NULL;
+
+	ok = g_volume_mount_finish (G_VOLUME (object), result, &error);
+	if (ok) {
+		rb_debug ("volume remounted successfully");
+	} else {
+		g_warning ("Unable to remount MTP device: %s", error->message);
+		g_error_free (error);
+	}
+	g_object_unref (object);
+}
+#endif
+
 static void
 rb_mtp_source_dispose (GObject *object)
 {
@@ -345,6 +481,20 @@ rb_mtp_source_dispose (GObject *object)
 		priv->device_thread = NULL;
 	}
 
+#if defined(HAVE_GUDEV)
+	if (priv->remount_volume != NULL) {
+		rb_debug ("remounting gvfs volume for mtp device");
+		/* the callback will unref remount_volume */
+		g_volume_mount (priv->remount_volume,
+				G_MOUNT_MOUNT_NONE,
+				NULL,
+				NULL,
+				remount_done_cb,
+				NULL);
+		priv->remount_volume = NULL;
+	}
+#endif
+
 	db = get_db_for_source (source);
 
 	g_object_get (G_OBJECT (source), "entry-type", &entry_type, NULL);
@@ -366,9 +516,17 @@ rb_mtp_source_finalize (GObject *object)
 	g_hash_table_destroy (priv->artwork_request_map);
 	g_hash_table_destroy (priv->track_transfer_map);		/* probably need to destroy the tracks too.. */
 
-#if !defined(HAVE_GUDEV)
+#if defined(HAVE_GUDEV)
+	if (priv->udev_device) {
+		g_object_unref (G_OBJECT (priv->udev_device));
+	}
+#else
 	g_free (priv->udi);
 #endif
+	g_free (priv->manufacturer);
+	g_free (priv->device_version);
+	g_free (priv->model_name);
+	g_free (priv->serial);
 
 	G_OBJECT_CLASS (rb_mtp_source_parent_class)->finalize (object);
 }
@@ -385,16 +543,15 @@ impl_get_paned_key (RBBrowserSource *source)
 	return g_strdup (CONF_STATE_PANED_POSITION);
 }
 
+RBSource *
+rb_mtp_source_new (RBShell *shell,
+		   RBPlugin *plugin,
 #if defined(HAVE_GUDEV)
-RBSource *
-rb_mtp_source_new (RBShell *shell,
-		   LIBMTP_raw_device_t *device)
+		   GUdevDevice *udev_device,
 #else
-RBSource *
-rb_mtp_source_new (RBShell *shell,
-		   LIBMTP_raw_device_t *device,
-		   const char *udi)
+		   const char *udi,
 #endif
+		   LIBMTP_raw_device_t *device)
 {
 	RBMtpSource *source = NULL;
 	RhythmDBEntryType entry_type;
@@ -412,13 +569,16 @@ rb_mtp_source_new (RBShell *shell,
 	g_object_unref (db);
 
 	source = RB_MTP_SOURCE (g_object_new (RB_TYPE_MTP_SOURCE,
+					      "plugin", plugin,
 					      "entry-type", entry_type,
 					      "shell", shell,
 					      "visibility", TRUE,
 					      "volume", NULL,
 					      "source-group", RB_SOURCE_GROUP_DEVICES,
 					      "raw-device", device,
-#if !defined(HAVE_GUDEV)
+#if defined(HAVE_GUDEV)
+					      "udev-device", udev_device,
+#else
 					      "udi", udi,
 #endif
 					      NULL));
@@ -428,6 +588,33 @@ rb_mtp_source_new (RBShell *shell,
 	return RB_SOURCE (source);
 }
 
+static void
+update_free_space_cb (LIBMTP_mtpdevice_t *device, RBMtpSource *source)
+{
+	RBMtpSourcePrivate *priv = MTP_SOURCE_GET_PRIVATE (source);
+	LIBMTP_devicestorage_t *storage;
+	int ret;
+
+	ret = LIBMTP_Get_Storage (device, LIBMTP_STORAGE_SORTBY_NOTSORTED);
+	if (ret != 0) {
+		rb_mtp_thread_report_errors (priv->device_thread, FALSE);
+	}
+
+	/* probably need a lock for this.. */
+	priv->free_space = 0;
+	for (storage = device->storage; storage != NULL; storage = storage->next) {
+		priv->free_space += storage->FreeSpaceInBytes;
+	}
+}
+
+static void
+queue_free_space_update (RBMtpSource *source)
+{
+	RBMtpSourcePrivate *priv = MTP_SOURCE_GET_PRIVATE (source);
+	rb_mtp_thread_queue_callback (priv->device_thread,
+				      (RBMtpThreadCallback) update_free_space_cb, source, NULL);
+}
+
 static void
 entry_set_string_prop (RhythmDB *db,
 		       RhythmDBEntry *entry,
@@ -644,6 +831,18 @@ device_opened_idle (DeviceOpenedData *data)
 static gboolean
 device_open_failed_idle (RBMtpSource *source)
 {
+	/* libmtp doesn't give us a useful error message in this case, so
+	 * all we can offer is this generic message.
+	 */
+	RBMtpSourcePrivate *priv = MTP_SOURCE_GET_PRIVATE (source);
+	rb_error_dialog (NULL,
+			 _("Media player device error"),
+			 /* Translators: first %s is the device manufacturer,
+			  * second is the product name.
+			  */
+			 _("Unable to open the %s %s device"),
+			 priv->raw_device.device_entry.vendor,
+			 priv->raw_device.device_entry.product);
 	rb_source_delete_thyself (RB_SOURCE (source));
 	g_object_unref (source);
 	return FALSE;
@@ -657,8 +856,6 @@ mtp_device_open_cb (LIBMTP_mtpdevice_t *device, RBMtpSource *source)
 	DeviceOpenedData *data;
 
 	if (device == NULL) {
-		rb_mtp_thread_report_errors (priv->device_thread, TRUE);
-
 		/* can't delete the source on this thread, so move it to the main thread */
 		g_idle_add ((GSourceFunc) device_open_failed_idle, g_object_ref (source));
 		return;
@@ -678,6 +875,25 @@ mtp_device_open_cb (LIBMTP_mtpdevice_t *device, RBMtpSource *source)
 		data->name = g_strdup (_("Digital Audio Player"));
 	}
 
+	/* get some other device information that doesn't change */
+	priv->manufacturer = LIBMTP_Get_Manufacturername (device);
+	priv->device_version = LIBMTP_Get_Deviceversion (device);
+	priv->model_name = LIBMTP_Get_Modelname (device);
+	priv->serial = LIBMTP_Get_Serialnumber (device);
+
+	/* calculate the device capacity */
+	priv->capacity = 0;
+	if (LIBMTP_Get_Storage (device, LIBMTP_STORAGE_SORTBY_NOTSORTED) == 0) {
+		LIBMTP_devicestorage_t *storage;
+		for (storage = device->storage;
+		     storage != NULL;
+		     storage = storage->next) {
+			priv->capacity += storage->MaxCapacity;
+		}
+	}
+
+	update_free_space_cb (device, RB_MTP_SOURCE (source));
+
 	/* figure out the set of formats supported by the device */
 	if (LIBMTP_Get_Supported_Filetypes (device, &data->types, &data->num_types) != 0) {
 		rb_mtp_thread_report_errors (priv->device_thread, FALSE);
@@ -864,6 +1080,7 @@ static gboolean
 impl_track_added (RBRemovableMediaSource *source,
 		  RhythmDBEntry *entry,
 		  const char *dest,
+		  guint64 filesize,
 		  const char *mimetype)
 {
 	LIBMTP_track_t *track = NULL;
@@ -894,6 +1111,7 @@ impl_track_added (RBRemovableMediaSource *source,
 		artdata->entry = rhythmdb_entry_ref (mtp_entry);
 		g_idle_add ((GSourceFunc) request_album_art_idle, artdata);
 	}
+	queue_free_space_update (RB_MTP_SOURCE (source));
 	return FALSE;
 }
 
@@ -912,6 +1130,7 @@ impl_track_add_error (RBRemovableMediaSource *source,
 	} else {
 		rb_debug ("track-add-error called, but can't find a track for dest URI %s", dest);
 	}
+
 	return TRUE;
 }
 
@@ -945,8 +1164,10 @@ prepare_encoder_sink_cb (RBEncoderFactory *factory,
 	db = get_db_for_source (source);
 	entry = rhythmdb_entry_lookup_by_id (db, track_id);
 	g_object_unref (db);
-	if (entry == NULL)
+	if (entry == NULL) {
+		g_free (extension);
 		return;
+	}
 
 	track = LIBMTP_new_track_t ();
 	track->title = rhythmdb_entry_dup_string (entry, RHYTHMDB_PROP_TITLE);
@@ -1042,10 +1263,129 @@ artwork_notify_cb (RhythmDB *db,
 	pixbuf = GDK_PIXBUF (g_value_get_object (metadata));
 
 	rb_mtp_thread_set_album_image (priv->device_thread, album_name, pixbuf);
+	queue_free_space_update (source);
 
 	g_object_unref (pixbuf);		/* ? */
 }
 
+static guint64
+impl_get_capacity	(RBMediaPlayerSource *source)
+{
+	RBMtpSourcePrivate *priv = MTP_SOURCE_GET_PRIVATE (source);
+	return priv->capacity;
+}
+
+static guint64
+impl_get_free_space	(RBMediaPlayerSource *source)
+{
+	RBMtpSourcePrivate *priv = MTP_SOURCE_GET_PRIVATE (source);
+	/* probably need a lock for this */
+	return priv->free_space;
+}
+
+static void
+impl_show_properties (RBMediaPlayerSource *source, GtkWidget *info_box, GtkWidget *notebook)
+{
+	RBMtpSourcePrivate *priv = MTP_SOURCE_GET_PRIVATE (source);
+	GtkBuilder *builder;
+	GtkWidget *widget;
+	GHashTableIter iter;
+	gpointer key, value;
+	int num_podcasts;
+	char *device_name;
+	char *builder_file;
+	RBPlugin *plugin;
+	char *text;
+	GList *output_formats;
+	GList *t;
+	GString *str;
+
+	g_object_get (source, "plugin", &plugin, NULL);
+	builder_file = rb_plugin_find_file (plugin, "mtp-info.ui");
+	g_object_unref (plugin);
+
+	if (builder_file == NULL) {
+		g_warning ("Couldn't find mtp-info.ui");
+		return;
+	}
+
+	builder = rb_builder_load (builder_file, NULL);
+	g_free (builder_file);
+
+	if (builder == NULL) {
+		rb_debug ("Couldn't load mtp-info.ui");
+		return;
+	}
+
+	/* 'basic' tab stuff */
+
+	widget = GTK_WIDGET (gtk_builder_get_object (builder, "mtp-basic-info"));
+	gtk_box_pack_start (GTK_BOX (info_box), widget, TRUE, TRUE, 0);
+
+	widget = GTK_WIDGET (gtk_builder_get_object (builder, "entry-mtp-name"));
+	g_object_get (source, "name", &device_name, NULL);
+	gtk_entry_set_text (GTK_ENTRY (widget), device_name);
+	g_free (device_name);
+	g_signal_connect (widget, "focus-out-event",
+			  (GCallback)rb_mtp_source_name_changed_cb, source);
+
+	num_podcasts = 0;
+	g_hash_table_iter_init (&iter, priv->entry_map);
+	while (g_hash_table_iter_next (&iter, &key, &value)) {
+		LIBMTP_track_t *track = value;
+		if (g_strcmp0 (track->genre, "Podcast") == 0) {
+			num_podcasts++;
+		}
+	}
+
+	widget = GTK_WIDGET (gtk_builder_get_object (builder, "mtp-num-tracks"));
+	text = g_strdup_printf ("%d", g_hash_table_size (priv->entry_map) - num_podcasts);
+	gtk_label_set_text (GTK_LABEL (widget), text);
+	g_free (text);
+
+	widget = GTK_WIDGET (gtk_builder_get_object (builder, "mtp-num-podcasts"));
+	text = g_strdup_printf ("%d", num_podcasts);
+	gtk_label_set_text (GTK_LABEL (widget), text);
+	g_free (text);
+
+	widget = GTK_WIDGET (gtk_builder_get_object (builder, "mtp-num-playlists"));
+	text = g_strdup_printf ("%d", 0);						/* correct, but wrong */
+	gtk_label_set_text (GTK_LABEL (widget), text);
+	g_free (text);
+
+	/* 'advanced' tab stuff */
+	widget = GTK_WIDGET (gtk_builder_get_object (builder, "mtp-advanced-tab"));
+	gtk_notebook_append_page (GTK_NOTEBOOK (notebook), widget, gtk_label_new (_("Advanced")));
+
+	widget = GTK_WIDGET (gtk_builder_get_object (builder, "label-mtp-model-value"));
+	gtk_label_set_text (GTK_LABEL (widget), priv->model_name);
+
+	widget = GTK_WIDGET (gtk_builder_get_object (builder, "label-serial-number-value"));
+	gtk_label_set_text (GTK_LABEL (widget), priv->serial);
+
+	widget = GTK_WIDGET (gtk_builder_get_object (builder, "label-firmware-version-value"));
+	gtk_label_set_text (GTK_LABEL (widget), priv->device_version);
+
+	widget = GTK_WIDGET (gtk_builder_get_object (builder, "label-manufacturer-value"));
+	gtk_label_set_text (GTK_LABEL (widget), priv->manufacturer);
+
+	str = g_string_new ("");
+	output_formats = rb_removable_media_source_get_format_descriptions (RB_REMOVABLE_MEDIA_SOURCE (source));
+	for (t = output_formats; t != NULL; t = t->next) {
+		if (t != output_formats) {
+			g_string_append (str, "\n");
+		}
+		g_string_append (str, t->data);
+	}
+	rb_list_deep_free (output_formats);
+
+	widget = GTK_WIDGET (gtk_builder_get_object (builder, "label-audio-formats-value"));
+	gtk_label_set_text (GTK_LABEL (widget), str->str);
+	g_string_free (str, TRUE);
+
+	g_object_unref (builder);
+}
+
 static void
 prepare_source (RBMtpSource *source, const char *stream_uri, GObject *src)
 {
@@ -1091,3 +1431,52 @@ prepare_encoder_source_cb (RBEncoderFactory *factory,
 	prepare_source (source, stream_uri, src);
 }
 
+#if defined(HAVE_GUDEV)
+
+static GMount *
+find_mount_for_device (GUdevDevice *device)
+{
+	GMount *mount = NULL;
+	const char *device_file;
+	GVolumeMonitor *volmon;
+	GList *mounts;
+	GList *i;
+
+	device_file = g_udev_device_get_device_file (device);
+	if (device_file == NULL) {
+		return NULL;
+	}
+
+	volmon = g_volume_monitor_get ();
+	mounts = g_volume_monitor_get_mounts (volmon);
+	g_object_unref (volmon);
+
+	for (i = mounts; i != NULL; i = i->next) {
+		GVolume *v;
+
+		v = g_mount_get_volume (G_MOUNT (i->data));
+		if (v != NULL) {
+			char *devname = NULL;
+			gboolean match;
+
+			devname = g_volume_get_identifier (v, G_VOLUME_IDENTIFIER_KIND_UNIX_DEVICE);
+			g_object_unref (v);
+			if (devname == NULL)
+				continue;
+
+			match = g_str_equal (devname, device_file);
+			g_free (devname);
+
+			if (match) {
+				mount = G_MOUNT (i->data);
+				g_object_ref (G_OBJECT (mount));
+				break;
+			}
+		}
+	}
+	g_list_foreach (mounts, (GFunc)g_object_unref, NULL);
+	g_list_free (mounts);
+	return mount;
+}
+
+#endif
