@@ -1,6 +1,4 @@
 /*
- *  arch-tag: Header for Rhythmbox playlist management object
- *
  *  Copyright (C) 2003,2004 Colin Walters <walters@verbum.org>
  *
  *  This program is free software; you can redistribute it and/or modify
@@ -8,6 +6,14 @@
  *  the Free Software Foundation; either version 2 of the License, or
  *  (at your option) any later version.
  *
+ *  The Rhythmbox authors hereby grant permission for non-GPL compatible
+ *  GStreamer plugins to be used and distributed together with GStreamer
+ *  and Rhythmbox. This permission is above and beyond the permissions granted
+ *  by the GPL license by which Rhythmbox is covered. If you modify this code
+ *  you may extend this exception to your version of the code, but you are not
+ *  obligated to do so. If you do not wish to do so, delete this exception
+ *  statement from your version.
+ *
  *  This program is distributed in the hope that it will be useful,
  *  but WITHOUT ANY WARRANTY; without even the implied warranty of
  *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
@@ -15,7 +21,7 @@
  *
  *  You should have received a copy of the GNU General Public License
  *  along with this program; if not, write to the Free Software
- *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
+ *  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301  USA.
  *
  */
 
@@ -25,8 +31,6 @@
 #include "rb-source.h"
 #include "rhythmdb.h"
 #include "rb-sourcelist.h"
-#include "rb-library-source.h"
-#include "rb-iradio-source.h"
 
 G_BEGIN_DECLS
 
@@ -39,61 +43,92 @@ G_BEGIN_DECLS
 
 typedef enum
 {
-	RB_PLAYLIST_MANAGER_ERROR_PARSE
+	RB_PLAYLIST_MANAGER_ERROR_PARSE,
+	RB_PLAYLIST_MANAGER_ERROR_PLAYLIST_EXISTS,
+	RB_PLAYLIST_MANAGER_ERROR_PLAYLIST_NOT_FOUND
 } RBPlaylistManagerError;
 
 #define RB_PLAYLIST_MANAGER_ERROR rb_playlist_manager_error_quark ()
 
 GQuark rb_playlist_manager_error_quark (void);
 
+typedef struct _RBPlaylistManager RBPlaylistManager;
+typedef struct _RBPlaylistManagerClass RBPlaylistManagerClass;
+
 typedef struct RBPlaylistManagerPrivate RBPlaylistManagerPrivate;
 
-typedef struct
+struct _RBPlaylistManager
 {
 	GObject parent;
 
 	RBPlaylistManagerPrivate *priv;
-} RBPlaylistManager;
+};
 
-typedef struct
+struct _RBPlaylistManagerClass
 {
 	GObjectClass parent_class;
 
 	/* signals */
-	void	(*playlist_added) (RBSource *source);
-	void	(*playlist_created) (RBSource *source);
-	void	(*load_start) (void);
-	void	(*load_finish) (void);
-} RBPlaylistManagerClass;
+	void	(*playlist_added) (RBPlaylistManager *manager, RBSource *source);
+	void	(*playlist_created) (RBPlaylistManager *manager, RBSource *source);
+	void	(*load_start) (RBPlaylistManager *manager);
+	void	(*load_finish) (RBPlaylistManager *manager);
+};
+
+typedef enum
+{
+	RB_PLAYLIST_EXPORT_TYPE_UNKNOWN,
+	RB_PLAYLIST_EXPORT_TYPE_M3U,
+	RB_PLAYLIST_EXPORT_TYPE_PLS,
+	RB_PLAYLIST_EXPORT_TYPE_XSPF,
+} RBPlaylistExportType;
 
 GType			rb_playlist_manager_get_type	(void);
 
 RBPlaylistManager *	rb_playlist_manager_new		(RBShell *shell,
-							 RBSourceList *sourcelist,
-							 RBLibrarySource *libsource,
-							 RBIRadioSource *iradio_source);
+						 	 RBSourceList *sourcelist,
+							 const char *playlists_file);
 
 void			rb_playlist_manager_shutdown	(RBPlaylistManager *mgr);
 gboolean 		rb_playlist_manager_parse_file	(RBPlaylistManager *mgr,
 							 const char *uri,
 							 GError **error);
 
-void			rb_playlist_manager_set_source	(RBPlaylistManager *mgr,
-							 RBSource *player);
-
-void			rb_playlist_manager_load_legacy_playlists (RBPlaylistManager *mgr);
-
 void			rb_playlist_manager_load_playlists (RBPlaylistManager *mgr);
 
-void			rb_playlist_manager_save_playlists_async (RBPlaylistManager *mgr, gboolean force);
-void			rb_playlist_manager_save_playlists (RBPlaylistManager *mgr, gboolean force);
+gboolean		rb_playlist_manager_save_playlists (RBPlaylistManager *mgr,
+							    gboolean force);
 
 RBSource *		rb_playlist_manager_new_playlist (RBPlaylistManager *mgr,
 							  const char *suggested_name,
 							  gboolean automatic);
 RBSource *		rb_playlist_manager_new_playlist_from_selection_data (RBPlaylistManager *mgr,
                                                                               GtkSelectionData *data);
-GList *			rb_playlist_manager_get_playlists (RBPlaylistManager *manager);
+
+GList *			rb_playlist_manager_get_playlists (RBPlaylistManager *mgr);
+
+gboolean		rb_playlist_manager_get_playlist_names (RBPlaylistManager *mgr,
+								gchar ***playlists,
+								GError **error);
+gboolean		rb_playlist_manager_create_static_playlist (RBPlaylistManager *mgr,
+								    const gchar *name,
+								    GError **error);
+gboolean		rb_playlist_manager_delete_playlist (RBPlaylistManager *mgr,
+							     const gchar *name,
+							     GError **error);
+gboolean		rb_playlist_manager_add_to_playlist (RBPlaylistManager *mgr,
+							     const gchar *name,
+							     const gchar *uri,
+							     GError **error);
+gboolean		rb_playlist_manager_remove_from_playlist (RBPlaylistManager *mgr,
+								  const gchar *name,
+								  const gchar *uri,
+								  GError **error);
+gboolean		rb_playlist_manager_export_playlist (RBPlaylistManager *mgr,
+							     const gchar *name,
+							     const gchar *uri,
+							     gboolean m3u_format,
+							     GError **error);
 
 G_END_DECLS
 
