@@ -7,7 +7,7 @@
  *  the Free Software Foundation; either version 2 of the License, or
  *  (at your option) any later version.
  *
- *  The Rhythmbox authors hereby grants permission for non-GPL compatible
+ *  The Rhythmbox authors hereby grant permission for non-GPL compatible
  *  GStreamer plugins to be used and distributed together with GStreamer
  *  and Rhythmbox. This permission is above and beyond the permissions granted
  *  by the GPL license by which Rhythmbox is covered. If you modify this code
@@ -53,8 +53,7 @@
 
 static void rb_missing_files_source_class_init (RBMissingFilesSourceClass *klass);
 static void rb_missing_files_source_init (RBMissingFilesSource *source);
-static GObject *rb_missing_files_source_constructor (GType type, guint n_construct_properties,
-						     GObjectConstructParam *construct_properties);
+static void rb_missing_files_source_constructed (GObject *object);
 static void rb_missing_files_source_dispose (GObject *object);
 static void rb_missing_files_source_set_property (GObject *object,
 						  guint prop_id,
@@ -93,7 +92,7 @@ rb_missing_files_source_class_init (RBMissingFilesSourceClass *klass)
 	RBSourceClass *source_class = RB_SOURCE_CLASS (klass);
 
 	object_class->dispose = rb_missing_files_source_dispose;
-	object_class->constructor = rb_missing_files_source_constructor;
+	object_class->constructed = rb_missing_files_source_constructed;
 
 	object_class->set_property = rb_missing_files_source_set_property;
 	object_class->get_property = rb_missing_files_source_get_property;
@@ -101,7 +100,6 @@ rb_missing_files_source_class_init (RBMissingFilesSourceClass *klass)
 	source_class->impl_can_browse = (RBSourceFeatureFunc) rb_false_function;
 	source_class->impl_get_entry_view = impl_get_entry_view;
 	source_class->impl_can_rename = (RBSourceFeatureFunc) rb_false_function;
-	source_class->impl_can_search = (RBSourceFeatureFunc) rb_false_function;
 
 	source_class->impl_can_cut = (RBSourceFeatureFunc) rb_false_function;
 	source_class->impl_can_delete = (RBSourceFeatureFunc) rb_true_function;
@@ -130,7 +128,7 @@ rb_missing_files_source_init (RBMissingFilesSource *source)
 
 	gtk_icon_size_lookup (RB_SOURCE_ICON_SIZE, &size, NULL);
 	pixbuf = gtk_icon_theme_load_icon (gtk_icon_theme_get_default (),
-					   GTK_STOCK_MISSING_IMAGE,
+					   "dialog-warning",
 					   size,
 					   0, NULL);
 	rb_source_set_pixbuf (RB_SOURCE (source), pixbuf);
@@ -139,22 +137,18 @@ rb_missing_files_source_init (RBMissingFilesSource *source)
 	}
 }
 
-static GObject *
-rb_missing_files_source_constructor (GType type, guint n_construct_properties,
-				     GObjectConstructParam *construct_properties)
+static void
+rb_missing_files_source_constructed (GObject *object)
 {
 	GObject *shell_player;
 	RBMissingFilesSource *source;
-	RBMissingFilesSourceClass *klass;
 	RBShell *shell;
 	GPtrArray *query;
 	RhythmDBQueryModel *model;
 	RhythmDBEntryType entry_type;
 
-	klass = RB_MISSING_FILES_SOURCE_CLASS (g_type_class_peek (RB_TYPE_MISSING_FILES_SOURCE));
-
-	source = RB_MISSING_FILES_SOURCE (G_OBJECT_CLASS (rb_missing_files_source_parent_class)->
-			constructor (type, n_construct_properties, construct_properties));
+	RB_CHAIN_GOBJECT_METHOD (rb_missing_files_source_parent_class, constructed, object);
+	source = RB_MISSING_FILES_SOURCE (object);
 
 	g_object_get (source,
 		      "shell", &shell,
@@ -208,8 +202,6 @@ rb_missing_files_source_constructor (GType type, guint n_construct_properties,
 
 	g_object_set (source, "query-model", model, NULL);
 	g_object_unref (model);
-
-	return G_OBJECT (source);
 }
 
 static void
