@@ -1,6 +1,4 @@
 /*
- *  arch-tag: Header for removable media source object
- *
  *  Copyright (C) 2005 James Livingston  <doclivingston@gmail.com>
  *
  *  This program is free software; you can redistribute it and/or modify
@@ -43,12 +41,15 @@ G_BEGIN_DECLS
 #define RB_IS_REMOVABLE_MEDIA_SOURCE_CLASS(k)  (G_TYPE_CHECK_CLASS_TYPE ((k), RB_TYPE_REMOVABLE_MEDIA_SOURCE))
 #define RB_REMOVABLE_MEDIA_SOURCE_GET_CLASS(o) (G_TYPE_INSTANCE_GET_CLASS ((o), RB_TYPE_REMOVABLE_MEDIA_SOURCE, RBRemovableMediaSourceClass))
 
-typedef struct
+typedef struct _RBRemovableMediaSource RBRemovableMediaSource;
+typedef struct _RBRemovableMediaSourceClass RBRemovableMediaSourceClass;
+
+struct _RBRemovableMediaSource
 {
 	RBBrowserSource parent;
-} RBRemovableMediaSource;
+};
 
-typedef struct
+struct _RBRemovableMediaSourceClass
 {
 	RBBrowserSourceClass parent;
 
@@ -60,6 +61,7 @@ typedef struct
 	gboolean	(*impl_track_added)	(RBRemovableMediaSource *source,
 						 RhythmDBEntry *entry,
 						 const char *uri,
+						 guint64 dest_size,
 						 const char *mimetype);
 	gboolean	(*impl_track_add_error) (RBRemovableMediaSource *source,
 						 RhythmDBEntry *entry,
@@ -67,7 +69,7 @@ typedef struct
 						 GError *error);
 	gboolean	(*impl_should_paste)	(RBRemovableMediaSource *source,
 						 RhythmDBEntry *entry);
-} RBRemovableMediaSourceClass;
+};
 
 typedef gboolean	(*RBRemovableMediaSourceShouldPasteFunc) (RBRemovableMediaSource *source,
 								  RhythmDBEntry *entry);
@@ -81,18 +83,19 @@ char*		rb_removable_media_source_build_dest_uri 	(RBRemovableMediaSource *source
 void		rb_removable_media_source_track_added		(RBRemovableMediaSource *source,
 								 RhythmDBEntry *entry,
 								 const char *uri,
+								 guint64 filesize,
 								 const char *mimetype);
 void		rb_removable_media_source_track_add_error	(RBRemovableMediaSource *source,
 								 RhythmDBEntry *entry,
 								 const char *uri,
 								 GError *error);
 GList *		rb_removable_media_source_get_mime_types	(RBRemovableMediaSource *source);
+GList *		rb_removable_media_source_get_format_descriptions (RBRemovableMediaSource *source);
 gboolean	rb_removable_media_source_should_paste		(RBRemovableMediaSource *source,
 								 RhythmDBEntry *entry);
 gboolean        rb_removable_media_source_should_paste_no_duplicate (RBRemovableMediaSource *source,
 								     RhythmDBEntry *entry);
 
-
 G_END_DECLS
 
 #endif /* __RB_REMOVABLE_MEDIA_SOURCE_H */
